import peasy.*; //PeasyCamライブラリをインストールしてインポート
PeasyCam cam;

PImage loadPng;
PShape shSnow, shGround;
ArrayList<PVector> snowFrakes;
FloatList groundHeights;
FloatList velocities;

boolean capture = false;

void setup() {
  size(1024, 768, P3D);
  cam = new PeasyCam(this, 700);
  //blendMode(ADD);
  //標高タイル画像を読み込む
  loadPng = loadImage("10-906-404.png");
  
  float pixMargin = 5.0;
  
  //雪のパーティクル
  snowFrakes = new ArrayList<PVector>();
  //落下速度の配列
  velocities = new FloatList();
  //雪を表す点群
  shSnow = createShape();
  shSnow.beginShape(POINTS);
  shSnow.strokeWeight(2);
  shSnow.stroke(255,255,255);
  for (int j=0; j<loadPng.height; j++) {
    for (int i=0; i<loadPng.width; i++) {
      //間引き50%
      if(random(0,1) < 0.5) continue;
      float frakeX = (i - loadPng.width /2) * pixMargin;
      float frakeY = (j - loadPng.height /2) * pixMargin;
      PVector snowFrake = new PVector( frakeX, frakeY, 400 + random(-100,100) );
      float velocity = random(-1,-0.5); 
      snowFrakes.add(snowFrake);
      velocities.append(velocity);
      shSnow.vertex(snowFrake.x, snowFrake.y, snowFrake.z);
    } 
  }
  shSnow.endShape();

  //標高の配列(落下点の高さ計算用)
  groundHeights = new FloatList();
  for (int j=0; j<loadPng.height; j++) {
    for (int i=0; i<loadPng.width; i++) {      
      color pxColor = loadPng.get(i, j);
      float red = red(pxColor);
      float green = green(pxColor);
      float blue = blue(pxColor);
      float z = (red * 256 * 256 + green * 256 + blue) *0.0005;
      groundHeights.append(z);
    }
  }
  
  //地形のメッシュ
  shGround = createShape();
  shGround.beginShape(QUADS);
  shGround.strokeWeight(3);
  shGround.stroke(0, 192, 255, 64);
  shGround.noFill();
  color pxColor;
  float red, green, blue, x, y, z;
  float zScale = 0.0005;
  for (int j=0; j<loadPng.height-1; j++) {
    for (int i=0; i<loadPng.width-1; i++) {

      pxColor = loadPng.get(i, j);
      red = red(pxColor);
      green = green(pxColor);
      blue = blue(pxColor);
      x = (i - loadPng.width /2) * pixMargin;
      y = (j - loadPng.height /2) * pixMargin;
      z = (red * 256 * 256 + green * 256 + blue) *zScale;
      shGround.vertex(x, y, z);

      pxColor = loadPng.get(i+1, j);
      red = red(pxColor);
      green = green(pxColor);
      blue = blue(pxColor);
      x = (i+1 - loadPng.width /2) * pixMargin;
      y = (j - loadPng.height /2) * pixMargin;
      z = (red * 256 * 256 + green * 256 + blue) *zScale;
      shGround.vertex(x, y, z);

      pxColor = loadPng.get(i+1, j+1);
      red = red(pxColor);
      green = green(pxColor);
      blue = blue(pxColor);
      x = (i+1 - loadPng.width /2) * pixMargin;
      y = (j+1 - loadPng.height /2) * pixMargin;
      z = (red * 256 * 256 + green * 256 + blue) *zScale;
      shGround.vertex(x, y, z);

      pxColor = loadPng.get(i, j+1);
      red = red(pxColor);
      green = green(pxColor);
      blue = blue(pxColor);
      x = (i - loadPng.width /2) * pixMargin;
      y = (j+1 - loadPng.height /2) * pixMargin;
      z = (red * 256 * 256 + green * 256 + blue) *zScale;
      shGround.vertex(x, y, z);
    }
  }
  shGround.endShape();
  
  
  
}

void draw() {
  background(0);
  
  //雪のパーティクルの位置を更新
  for (int i=0; i<shSnow.getVertexCount(); i++) {
      PVector v = shSnow.getVertex(i);
      PVector snowFrake = snowFrakes.get(i);
      float ground = groundHeights.get(i);
      float velocity = velocities.get(i);
      v.x = snowFrake.x + random(-2,2);
      v.y = snowFrake.y + random(-2,2);
      v.z += velocity;
      if(v.z < ground){
        v.z = 400 + random(-100,100);
      }
      shSnow.setVertex(i,v);
  }

  shape(shSnow);
  shape(shGround);
  
  if(capture){
    saveFrame("data/capture/#####.png");
  }
}


void keyPressed(){
  if(key =='s'){
    capture = !capture;
  }
}
